function [ quad, pvtol, rotor, rho ] = Coefficients(Upsilon)
%   [ quadrotor, vtol, rotor ] = Coefficients(Upsilon)
%   Defines the rotor and airframe coefficents for the   
%   Draganflyer XPro quadrotor and a conceptual PVTOL aircraft
%   based on measurements from Martinez (2007).
%
%   Inputs:
%      Upsilon = outwards rotortilt angle
%
%   Outputs:
%      quadrotor = Draganflyer XPro quadrotor coefficients data structure
%      pvtol = conceptual PVTOL aircraft coefficients data structure
%      rotor = Draganflyer XPro quadrotor rotor coefficients data structure
%      rho = atmosphere air density (kg/m3) at sea level
%  
%   BiBTeX References:
%   @MastersThesis{MartinezMSC2007,
%    Title       = {Modelling of the Flight Dynamics of a Quadrotor Helicopter},
%    Author      = {Mart\'{\i}nez, V. M.},
%    School      = {Cranfield University},
%    Year        = {2007}, 
%    Address     = {Bedfordshire, U.K.},
%    File        = {http://public.cranfield.ac.uk/eh3081/reports/VicenteMartinezMSc2007.pdf}
%   }
%   @Book{SeddonBook1990,
%    Title       = {Basic Helicopter Aerodynamics},
%    Author      = {Seddon, J.},
%    Publisher   = {BSP Professional},
%    Year        = {1990},
%    Address     = {Oxford, U.K.}
%    }
%    @Book{BramwellBook2001,
%     Title      = {Bramwell's Helicopter Dynamics},
%     Author     = {Bramwell, A. R. S. and Done, G. and Balmford, D.},
%     Publisher  = {Butterworth-Heinemann},
%     Year       = {2001},
%     Address    = {Oxford, U.K.},
%     Edition    = {2nd}
%    }

%   (c) James Whidborne
%   Cranfield University, 15 August 2020
 
if nargin==0, Upsilon=1; end

%% atmosphere
rho= 1.225; % air density (kg/m3)


%% rotor parameters (Martinez p 133-143 & 274)

rotor.a = 5.5;        % Aerofoil lift slope 
rotor.R = 0.258;      % Rotor disc radius (including the hinge offset)
c = 0.04;             % Mean chord (0.032 on p 135)
b = 2;                % Number of blades
rotor.sigma = b*c/(pi*rotor.R); % Rotor solidity
rotor.theta0 = 0.37;  % Blade pitch at the root
rotor.theta1 = -0.09; % Twist (Seddon p 27 theta = theta_0+r theta_tw,  r \in [0,1])
rotor.theta = rotor.theta0+ rotor.theta1*3/4; % twist at 3/4 radius
% drag coefficient is constant in Seddon, and is quadratic in
% Bramwell (ed2) p 135. Here we take the linear expression obtained
% by Martinez Cd = Cd0 + Cd1*alpha
rotor.Cd0=0.05;       % cd = 0.05 + 0.7 alpha (Martinez p 143)
rotor.Cd1=0.7;        % 
 
%% quadrotor vehicle parameters (Martinez p 273-274)

% mass and inertias
quad.m = 2.356;            % total mass[kg]
Ix = 0.1676;               % inertia of x axis [kg.m^2]
Iy = 0.1676;               % inertia of y axis [kg.m^2]
Iz = 0.29743;              % inertia of z axis [kg.m^2] (should be at least 2I_x)
quad.J = diag([Ix;Iy;Iz]); % moment of inertia matrix
%quad.rBC = [0;0;0];       % vector from body frame to center of mass

% arm vector/rotor geometry
quad.l = 0.4534;           % armlength (Martinez p 273)
% arm vectors from B to Ri
quad.rBi = quad.l*[1 0 -1 0;
    0 1 0 -1;
    0 0 0 0];
% rotor rotation matrices from Body frame to ith rotor frame
RBi = zeros(3,3,4); % 3 rows, 3 cols, 4 pages
RBi(:,:,1) = DCM(Upsilon*[0;1;0]); 
RBi(:,:,2) = DCM(Upsilon*[-1;0;0]); 
RBi(:,:,3) = DCM(Upsilon*[0;-1;0]); 
RBi(:,:,4) = DCM(Upsilon*[1;0;0]); 
% rotor direction vectors for 4 rotors
quad.zi = squeeze(-RBi(:,3,:));% extract third column (i.e. negative z-direction)

% rotor inclination angle
%quad.Upsilon = Upsilon;    % not used - just to keep track of it

% rotor parameters
quad.rotor = rotor; % rotor parameters 

% Control Allocation matrix   Omega = CA u
% Maps the controls to the channels:
% u1 --> c(Omega1 - Omega3) : x-channel
% u2 --> c(-Omega2 + Omega4) : y-channel
% u3 --> b(Omega1 + Omega2 + Omega3 + Omega4) : z-channel
% u4 --> d(Omega1 - Omega2 + Omega3 - Omega4) : psi-channel
% CA= inv([ c  0 -c  0;
%           0 -c  0  c;
%           b  b  b  b;
%           -d d  -d d])
%  
b = 0.033; c = 0.2;  d = 0.01; % parameters of control allocation  
quad.CA = [-1/(2*c),         0,   1/(4*b), - 1/(4*d);
                  0,  -1/(2*c),   1/(4*b),   1/(4*d);
            1/(2*c),         0,   1/(4*b), - 1/(4*d);
                  0,   1/(2*c),   1/(4*b),   1/(4*d)];

% environment parameters 
quad.rho = rho; % air density (kg/m3)

%%  PVTOL vehicle airframe parameters   
% conceptual vehicle parameters - approx based on Draganflyer XPro
pvtol.m = 1.18; % mass  (kg) - half quad mass because only 2 rotors
pvtol.I = 0.0625;   % mass is reduced, so inertia is arbiratily set  
pvtol.l = 0.45;   % armlength (m)  
% rotor parameters
pvtol.rotor = rotor; % rotor parameters 
% environment parameters 
pvtol.rho = rho; % air density (kg/m3)

end

%% DCM (Rotation matrix)
function T=DCM(rpy)
% Direction Cosine Matrix
phi=rpy(1,:);theta=rpy(2,:);psi=rpy(3,:); %roll, pitch, yaw
%  Rotation matrix
T=[cos(theta).*cos(psi),   cos(theta).*sin(psi), -sin(theta);
    sin(phi).*sin(theta).*cos(psi) - cos(phi).*sin(psi),...
    sin(phi).*sin(theta).*sin(psi) + cos(phi).*cos(psi),...
    sin(phi).*cos(theta);
    cos(phi).*sin(theta).*cos(psi) + sin(phi).*sin(psi),...
    cos(phi).*sin(theta).*sin(psi) - sin(phi).*cos(psi),...
    cos(phi).*cos(theta)];
end
