function [A,B] = LinearizeAB(fun, x0, u0)
%LINEARIZEAB
%   [A,B] = LinearizeAB(fun, x0, u0)
%    Obtains a state space linear model from a nonlinear state space
%    model using a finite difference method 
%  
%    Inputs:
%     fun:  anonymous function that takes vector arguments of
%          state x of length n and control u and returns a vector of
%          length n state derivatives, xdot=fun(x,u)
%     x0: equilibrium (trim) point   (vector of length n)
%     u0: equilibrium (trim) point
%
%    Outputs:
%     A: matrix containing the gradient of fun at (x0,u0)
%    i.e. A=[df1/dx1, df1/dx2, df1/dx3 ...]
%           [df2/dx1, df2/dx2, df2/dx3 ...]
%           [...       ....     ....   ...]
%
%     B: matrix containing the gradient of fun at (x0,u0)
%    i.e. B=[df1/du1, df1/du2, df1/du3 ...]
%           [df2/du2, df2/du2, df2/du3 ...]
%           [...       ....     ....   ...]
%
%   Uses finite difference method - either 1st or 2nd order

%   (c) James Whidborne
%   Cranfield University, 26 April 2017

d=1e-9; % perturbation magnitude

%% A-matrix
nx=length(x0); % state dimension
A=zeros(nx,nx); %preallocate matrix
for i = 1 : nx % loop state variables
    % partial derivatives in row vector
    
    xd = x0; % original vector
    xd(i) = x0(i)+d; % +ve perturb ith term
    fd1 = feval( fun, xd, u0 ); %
    xd(i) = x0(i)-d; % -ve perturb ith term
    fd2 = feval( fun, xd, u0 ); %
    if(0) % 1st order method
        A(:,i)=(fd1-fd2)/(2*d);
    else% 2nd order method
        xd(i) = x0(i)+2*d; % +ve perturb ith term
        f2d1 = feval( fun, xd, u0 ); %
        xd(i) = x0(i)-2*d; % -ve perturb ith term
        f2d2 = feval( fun, xd, u0 ); %
        A(:,i)=(-f2d1+8*fd1-8*fd2+f2d2)/(12*d);
    end
    
    %% B-matrix
    if nargout==2
        nu=length(u0); % control dimension
        B=zeros(nx,nu);
        for i = 1 : length(u0)
            ud = u0; % original vector
            ud(i) = u0(i)+d; % +ve perturb ith term
            fd1 = feval( fun, x0, ud ); %
            ud(i) = u0(i)-d; % -ve perturb ith term
            fd2 = feval( fun, x0, ud  ); %
            if(0) % 1st order
                B(:,i)=(fd1-fd2)/(2*d);
            else% 2nd order
                ud(i) = u0(i)+2*d; % +ve perturb ith term
                f2d1 = feval( fun, x0, ud  ); %
                ud(i) = u0(i)-2*d; % -ve perturb ith term
                f2d2 = feval( fun, x0, ud  ); %
                B(:,i)=(-f2d1+8*fd1-8*fd2+f2d2)/(12*d);
            end
        end
    end
end
return


