function [ xvecdot ] = PvtolDynamics( xvec, uvec, pvtol, Upsilon )
%PVTOLROTORDYNAMICS
%   [xvecdot] = PvtolDynamics( xvec, uvec, pvtol, Upsilon )
%   Returns state derivatives for model of planar birotor VTOL dynamics in
%   the body axes
%
%   Inputs
%     state vector = xvec=[z,w,x,u,phi,omega]
%     control vector = xvec=[Omegal+Omegar,Omegal-Omegar,Vw]
%     pvtol = vehicle airframe coefficients structure (see Coefficients.m) 
%     Upsilon = rotor tilt angle (rad)

%   (c) James Whidborne
%   Cranfield University,  19 August 2020

w = xvec(2); u = xvec(4); theta = xvec(5); q = xvec(6); % state variables
Omegal = (uvec(1)+uvec(2))/2; Omegar = (uvec(1)-uvec(2))/2; Vw = uvec(3); % controls & disturbance
ql = q*pvtol.l; % q \ell
%
g = 9.81;           % gravitational constant (m/s^2)
rotor = pvtol.rotor;% rotor coefficients
m = pvtol.m;        % mass (kg)
ell = pvtol.l;      % arm length (m)
MoI = pvtol.I;      % moment of inertia (kg m^2)
A = pi*rotor.R^2;   % disc area (m^2)
rho = pvtol.rho;    % air density (kg/m^3)

% AoA and relative airspeed to rotors
VlT = Vw*sin(theta+Upsilon) - u*sin(Upsilon) - (w+ql)*cos(Upsilon);
VlH = Vw*cos(theta+Upsilon) - u*cos(Upsilon) + (w+ql)*sin(Upsilon);
alphal = atan2(VlT,VlH);
if(isnan( alphal)), alphal=0;end

VrT = Vw*sin(theta-Upsilon) + u*sin(Upsilon) - (w-ql)*cos(Upsilon);
VrH = Vw*cos(theta-Upsilon) - u*cos(Upsilon) - (w-ql)*sin(Upsilon);
alphar = atan2(VrT,VrH);
if(isnan( alphar)), alphar=0;end

Vl = sqrt(VlT^2 + VlH^2);
Vr = sqrt(VrT^2 + VrH^2);

% get thrusts and drages - solve using fzero
Tl  = fzero(@(x)rho*A*(Omegal*rotor.R)^2*RotorCT(x,alphal,Vl,Omegal,rho,rotor)-x, m*g/2);
Hl  = rho*A*(Omegal*rotor.R)^2*RotorCH(Tl,alphal,Vl,Omegal,rho,rotor);
Tr  = fzero(@(x)rho*A*(Omegar*rotor.R)^2*RotorCT(x,alphar,Vr,Omegar,rho,rotor)-x, m*g/2);
Hr  = rho*A*(Omegar*rotor.R)^2*RotorCH(Tr,alphar,Vr,Omegar,rho,rotor);

% equations of motion
udot = -q*w -g*sin(theta) - (Tl-Tr)*sin(Upsilon)/m + (Hl+Hr)*cos(Upsilon)/m;
wdot = q*u+ g*cos(theta) - (Tl+Tr)*cos(Upsilon)/m - (Hl-Hr)*sin(Upsilon)/m;
qdot = ( -(Tl-Tr)*ell*cos(Upsilon) - (Hl+ Hr)*ell*sin(Upsilon) )/MoI;
 
% navigation equations
xdot =  u*cos(theta) + w*sin(theta);
zdot =  -u*sin(theta) + w*cos(theta);
thetadot = q ;
  
% state derivative vector
 xvecdot = [zdot; wdot; xdot; udot; thetadot; qdot];
 
end

% Earth axis state derivatives [ z, zdot, x , xdot, theta, thetadot]
%      [zdot;
%      (-Tl*cos(theta+Upsilon)-Tr*cos(theta-Upsilon)-Hl*sin(theta+Upsilon)-Hr*sin(theta-Upsilon))/m + g;
%      xdot; %  
%      (-Tl*sin(theta+Upsilon)-Tr*sin(theta-Upsilon)+Hl*cos(theta+Upsilon)+Hr*cos(theta-Upsilon))/m;
%      q; %  
%      (-(Tl-Tr)*ell*cos(Upsilon)-(Hl+Hr)*ell*sin(Upsilon))/MoI] 

