function xdot=QuadrotorDynamics(x,Omega,quad,wind)
%QUADROTORDYNAMICS
%  xdot=QuadrotorDynamics(x,Omega,quad,wind)
%  Returns state derivatives for model of quadrotor dynamics ignoring 
%  the rotor gyroscopic effects and using the body axis derivatives  
%
%  Inputs
%   x = state vector:[xe ye ze u v w phi theta psi p q r]- state vector
%   Omega = control (4 rotor speeds)
%   quad = quadrotor coefficients structure (see Coefficients.m) 
%   wind = wind disturbance vector
 
%   (c) James Whidborne
%   Cranfield University, 18 August 2020
 

g=9.81;          % gravitational constant (m/s^2)
uvw = x(4:6);    % velocity  in body axes: u v w (m/s)
rpy = x(7:9);    % euler angles : roll, pitch, yaw (rad)
pqr = x (10:12); % rotation rates vector: p q r (rad/s) 

%% calculate Force and moments in body axis
mg = [0;0;quad.m*g]; % m g z_e

G = [DCM(rpy)*mg;
    zeros(3,1)]; % gravitational force and moment about centre of mass
[F,T] = aero(pqr,uvw,rpy,quad, wind,Omega); % rotor forces and moments about CoM
FT = [sum(F,2); sum(T,2)]+G; % add force and moment components

% Newton Euler equations
% for CoM not at location B
%  M=[p.m*eye(3), -p.m*cross(p.rBC);
%     p.m*cross(p.rBC), p.J-p.m*cross(p.rBC)*cross(p.rBC)];
%  C=[p.m*cross(pqr)*cross(pqr)*p.rBC;
%     cross(pqr)*(p.J-p.m*cross(p.rBC)*cross(p.rBC))*pqr];
% B located at CoM
M = [quad.m*eye(3), zeros(3,3);
    zeros(3,3),    quad.J]; % mass & Inertia matrix
C = [cross(pqr)*quad.m*eye(3)*uvw ; 
    cross(pqr)*quad.J*pqr]; % fictional terms
%
Xdot = M\(FT-C);        % Newton Euler equations
xyzdot = DCM(rpy)'*uvw; % map to earth axes rates d(x_e, y_e, z_e)/dt
uvwdot = Xdot(1:3);     % body velocity rates d(u,v,w)/dt
rpydot = KDE(rpy)*pqr;  % rates of Euler angles d(phi, theta, psi)/dt (roll, pitch, yaw)
pqrdot = Xdot(4:6);     % body rotation rates d(p,q,r)/dt

xdot = [xyzdot;  % position rates 
       uvwdot;   % uvw rates
       rpydot;   % Euler rates
       pqrdot];  % pqr rates
end


%% Aero forces and moments
function [F,M] = aero(pqr, uvw, rpy, p, wind, Omega)
% uvw = velocity  in body axes
% rpy = euler angles (roll, pitch, yaw)
% pqr = rotation rates vector
% p = vehicle parameters (quad from Coefficients.m))
% wind = wind vector
% Omega = rotor speeds

A = pi*p.rotor.R^2; % disc area
T0 = p.m*9.81/4;    % approx thrust at hover for fzero search initial value
F = zeros(3,4);     % forces of each rotor
M = zeros(3,4);     % moments of each rotor
Vai = zeros(3,4);   % velocity of ith rotor in wind axis
alpha = zeros(1,4); % angle of attack
T = zeros(1,4);     % thrust of each rotor
H = zeros(1,4);     % drag of each rotor
Q = zeros(3,4);     % torques of each rotor
V = zeros(1,4);     % airspeed at each rotor
for i=1:4 % step through the 4 rotors
    Vai(:,i) = DCM(rpy)*wind-(uvw+cross(pqr)*p.rBi(:,i));% air incident velocity  in body axis at each rotor
    V(i) = norm(Vai(:,i)); % airspeed
    if V(i)<2*eps % no wind hence vehicle is still relative to wind
        T(i) = fzero(@(x)p.rho*A*(Omega(i)*p.rotor.R)^2*RotorC(x,0,0,Omega(i),p.rho,p.rotor)-x,T0);% thrust (v=0, alpha=0)
        alpha(i) = 0; % zero incidence angle
        [~,CQ] = RotorCHQ(T(i),0,0,Omega(i),p.rho,p.rotor); % zero drag
        F(:,i) = T(i)*p.zi(:,i); % body axis force (zero drag)
    else
        alpha(i) = -asin(Vai(:,i)'*p.zi(:,i)/V(i));  % incident angle
        T(i) = fzero(@(x)p.rho*A*(Omega(i)*p.rotor.R)^2*RotorC(x,alpha(i),V(i),Omega(i),p.rho,p.rotor)-x,T0); % thrust
        [CH,CQ] = RotorCHQ(T(i),alpha(i),V(i),Omega(i),p.rho,p.rotor);
        H(i) = p.rho*A*(Omega(i)*p.rotor.R)^2*CH; % in-plane drag
        F(:,i) = T(i)*p.zi(:,i)+H(i)*Vai(:,i)/norm(Vai(:,i)); % force in body axis
    end
    Q(:,i) = (-1)^i*p.rho*A*Omega(i)^2*p.rotor.R^3*CQ*p.zi(:,i); % rotor torque
    M(:,i) = cross(p.rBi(:,i))*F(:,i) + Q(:,i); % moments about CoM in body frame
end
end

%% DCM (Rotation matrix)
function T = DCM(rpy)
% Direction Cosine Matrix
phi = rpy(1,:); theta = rpy(2,:); psi = rpy(3,:); %roll, pitch, yaw
%  Rotation matrix
T = [cos(theta).*cos(psi),   cos(theta).*sin(psi), -sin(theta);
    sin(phi).*sin(theta).*cos(psi) - cos(phi).*sin(psi),...
    sin(phi).*sin(theta).*sin(psi) + cos(phi).*cos(psi),...
    sin(phi).*cos(theta);
    cos(phi).*sin(theta).*cos(psi) + sin(phi).*sin(psi),...
    cos(phi).*sin(theta).*sin(psi) - sin(phi).*cos(psi),...
    cos(phi).*cos(theta)];
end


%% cross product operator
function S = cross(xyz)
% [xij yij zij]=x
xij = xyz(1); yij = xyz(2); zij = xyz(3);
%  cross product operator
S = [0  -zij yij;
    zij   0 -xij;
    -yij xij  0];
end

%%  pqr to Euler angle rates transform via the
%  kinematic differential equation
function kde = KDE(rpy)
%% rpy = [phi;theta;psi] (roll, pitch, yaw)
phi = rpy(1); theta = rpy(2); %psi = rpy(3);
kde = [1 sin(phi)*tan(theta)  cos(phi)*tan(theta);
    0            cos(phi)            -sin(phi);
    0 sin(phi)/cos(theta)  cos(phi)/cos(theta)];
end
