function [CT,CH,CQ,lambda]=RotorC(T,alpha,V,Omega,rho,rotor)
%ROTORC
%   [CT,CH,CQ]=RotorC(T,alpha,V,Omega,rho,rotor)
%   Returns rotor thrust and drag coefficients CT(T) and CH(T) and the 
%   blade torque coefficient CQ(T)
%   the subsequent thrust and drag are calculated from
%      T = rho*A*(Omega*rotor.R)^2*CT(T,alphar,V,Omega,rho,rotor)
%      H = rho*A*(Omega*rotor.R)^2*CH(T,alphar,V,Omega,rho,rotor)
%      Q = rho*A*Omega^2*rotor.R^3*CQ(T,alphar,V,Omega,rho,rotor)
%   NOTE that this is an implicit equation and must be solved iteratively
%   using fzero.m or similar
%
%   CT: rotor thrust coefficient
%   CH: rotor drag coefficient
%   CQ: blade torque coefficient
%
%   T: rotor thrust
%   alpha : angle of attack/wind incidence angle (rad)
%   V : incident windspeed (m/s)
%   Omega : rotor speed (rad/sec)
%   rho: air density (kg/m^3)
%   rotor: vehicle rotor coefficients structure (see Coefficients.m)
% References:
%   Bramwell, "Bramwells Helicopter Dynamics". Butterworth, 2nd ed., 2001
%   Seddon, "Basic Helicopter Aerodynamics". BSP Professional, 1990.


%   (c) James Whidborne
%   Cranfield University, 1 March 2020

A= pi*rotor.R^2;                   % disc area
mu=V*cos(alpha)./(rotor.R*Omega);  % advance ratio
if mu>0.5, mu=0.5;  end            %  max mu = 0.5 (Bramwell,  p. 96)


% calculate vh (Seddon[p52]: eq (5.4) Bramwell[p78]: eq (3.2) )
vhsquared= (T/(2*rho*A));
visquared = (sqrt( V.^4+4*vhsquared.^2) - V.^2)/2;
vi=sqrt(visquared);                          % induced downwash
lambda = (V.*sin(alpha) + vi)./(Omega*rotor.R);% rotor inflow  ratio - note
%different sign in Bramwell - this is the Seddon convention
% Seddon[p59]: eq(2.27) , Bramwell[p98]: eq (3.33)
CT = rotor.sigma/2 * rotor.a * (rotor.theta/3 * (1+3*mu.^2/2) - lambda/2);

% flapping coefficient Seddon: eq 5.50, , Bramwell[p107]: eq (3.56)
%a1=mu.*(8*rotor.theta0/3 - 2*lambda)./(1+3*mu.^2/2);% Seddon: eq 5.53
a1=mu.*(8*rotor.theta0/3 - 2*lambda)./(1-mu.^2/2);% Seddon: eq 5.50


% Bramwell[p 100]: eq (3.39)
%CH= rotor.a*rotor.sigma/2*(mu*delta/(2*rotor.a) + a1*rotor.theta/3 ...
%    + 3*lambda.*a1/4 - mu.*rotor.theta.*lambda/2 + mu.*a1.^2/4);
CH = rotor.a*rotor.sigma/2*(mu*rotor.Cd0/(2*rotor.a) + a1*rotor.theta/3 ...
    - 3*lambda.*a1/4 + mu.*rotor.theta.*lambda/2 + mu.*a1.^2/4); % corrected sign on lambda
% CH = rotor.sigma*delta* mu/4; % alternative expression
% Seddon p61 eq(5.32)

% Seddon p 62 eq (5.37)
%CQ  = rotor.sigma*rotor.Cd0*(1+3*mu^2)/8+lambda*CT-mu*CH; % this equation gives very poor results
% Prouty (1990) -incomplete?
CQ = rotor.sigma/4*(rotor.Cd0*(1+mu^2)/2 ...
    + rotor.Cd1/(1+3/2*mu^2)*(rotor.theta0*(1/2-19/36*mu^2+3/4*mu^4) ...
    +  rotor.theta1*(2/5-2/5*mu^2+mu^4/2) - (2-mu^2)/3*lambda));
end


