% Section5.m
%
% Generate figures for Section 5 of paper
% "The Effect of Rotor Tilt on the Stability and Gust Rejection Properties
%  of VTOL Multirotor Aircraft"
% by James Whidborne Arthur Mendez and Alastair Cooke

% (c) James Whidborne
% Cranfield University, 16 August 2020

disp(' ')
disp('----------------------')
disp('5 PVTOL Linear control')
disp('----------------------')
disp(' ')

% trim and linearize birotor VTOL  & design controller
% in constant  horizontal wind for a set of  Upsilon values

%% parameters & initial values
[~, pvtol, ~, ~] = Coefficients; % PVTOL coefficients
g = 9.81;     % gravitational constant (m/s^2)
Vw = 0;       %  constant  horizontal wind airspeed (m/s)
Upsilonset = [-5,0,5,10] % set of  Upsilon values

% initial trim guess values
Omegal0 = 150; Omegar0 = 150; theta0 = 0;

% initialize storage vectors
t = 0:0.02:12; %  time
N = length(t);
% lateral reference step response
Omegal_step = zeros(4,N);Omegar_step = zeros(4,N);
zpos_step = zeros(4,N);
xpos_step = zeros(4,N);theta_step = zeros(4,N);
% gust step response
Omegal_gust = zeros(4,N);Omegar_gust = zeros(4,N);
zpos_gust = zeros(4,N);
xpos_gust = zeros(4,N);theta_gust = zeros(4,N);

% options for trim residual minimization
trimoptions = optimset('TolFun',1e-7,'TolX',1e-7);

%% main loop
for i=1:length(Upsilonset)
    disp('----------------------')
    disp(['Upsilon = ',num2str(Upsilonset(i))])
    Upsilon=Upsilonset(i)*pi/180;
    xtrim0=[0;0;0;0;theta0;0]; utrim0=[Omegal0+Omegar0;Omegal0-Omegar0;Vw];% set initial trim points guess
    
    %trim & linearize
    [A,B,xtrim,utrim ] = PvtolTrimLin(pvtol,Vw,Upsilon,xtrim0,utrim0,trimoptions);
    Bu=B(:,1:2);    % extract control matrix
    Bw=B(:,3);      % extract disturbance matrix
    C=[1 0 0 0 0 0; % output matrix
        0 0 1 0 0 0;
        0 0 0 0 1 0];
    thetatrim=xtrim(5); Omegaltrim=(utrim(1)+utrim(2))/2;  Omegartrim=(utrim(1)-utrim(2))/2; % record trim
    
    %% LQR controller
    %Q=eye(6); R=eye(2)/2; Q(1,1)=100; % weighting matrices
    %Klqr=lqr(A,Bu,Q,R);  % optimal LQR regulator
    
    %% Pole Placement controller
    Pz=[-1.5,-0.6];    % vertical - based on Upsilon=10 lqr controller
    Px=[-5.5,-2.2,-1.5,-0.6]; % horizontal/pitch - based on Upsilon=10 lqr controller
    Kplacez=place(A(1:2,1:2),Bu(1:2,1),Pz); %  vertical channel
    Kplacex=place(A(3:6,3:6),Bu(3:6,2),Px); %  horizontal/pitch channel
    Kplace=[Kplacez, zeros(1,4);zeros(1,2),Kplacex];
    disp('Controller:')
    K=Kplace  % controller
    
    %% linear step response - not recorded in paper
    if(0)
        
        T = ss(A-Bu*K,Bu*K,C,0); % closed loop system
        %        disp('Closed loop eigenvalues:'), eig(T) % stability check
        
        % response to lateral position reference step
        tlin1 = 0:.01:10; % time vector
        [y] = step(T(1:3,2),tlin1); % step response
        %%
        
        figure(201) % plot
        subplot(2,1,1)
        plot(tlin1,y(:,1:2),'linewidth',lw)
        grid on
        xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
        ylabel('$x(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
        title('Response to unit lateral position step demand')
        hold on
        subplot(2,1,2)
        plot(tlin1,y(:,3)*180/pi,'linewidth',lw)
        grid on
        xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
        ylabel('$\theta(t)$ (deg)','Interpreter' ,'Latex','fontsize',fs)
        hold on
        
        % response to gust step
        Cw=[0 0 1 0 0 0;
            0 0 0 0 1 0]; % output x and theta
        Tw=ss(A-Bu*K,Bw,Cw,0);
        tlin2=0:.01:15;% time vector
        [yW]=step(Tw,tlin2);% step response
        figure(202)% plot response to gust step
        subplot(2,1,1)
        plot(tlin2,yW(:,1),'linewidth',lw)
        grid on
        xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
        ylabel('$x(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
        title('Response to unit horizontal wind step')
        hold on
        subplot(2,1,2)
        plot(tlin2,yW(:,2)*180/pi,'linewidth',lw)
        grid on
        xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
        ylabel('$\theta(t)$ (deg)','Interpreter' ,'Latex','fontsize',fs)
        hold on
    end%if
    
    %% nonlinear model simulations: 10 m lateral demand step response
    x0 = xtrim;          % initial stste
    %xref=zeros(12,1);   % state reference
    yref = [0;10;0];     % output reference
    d = 0;               % wind disturbance
    options = odeset('RelTol',1e-4,'AbsTol',1e-4);
    [t1,x_step] = ode45(@(t,x)PvtolSim(t, x, yref, utrim, K, pvtol, Upsilon),t,x0,options);
    % calculate control
    x_step = x_step';  % convert to column vector of states
    y_step = C*x_step; % output
    e = yref-y_step;   % output error
    u_step = K*[e(1,:);-x_step(2,:);e(2,:);-x_step(4,:);e(3,:);-x_step(6,:)]+utrim(1:2)'; % control
    
    Omegal_step(i,:) = (u_step(1,:)+u_step(2,:))/2;  % rotor speed left
    Omegar_step(i,:) = (u_step(1,:)-u_step(2,:))/2;  % rotor speed right
    zpos_step(i,:) = x_step(1,:);                    % vertical position
    xpos_step(i,:) = x_step(3,:);                    % lateral position
    theta_step(i,:) = x_step(5,:);                   % roll angle
    
    %% nonlinear model simulations: 5 m/s wind disturbance step response
    x0 = xtrim;              % initial stste
    xref = zeros(12,1);      % state reference
    yref = [0;0;0];          % output reference
    d = 5;                   % wind disturbance constant
    utrimd = [utrim(1:2),d]; % put disturbance into trim
    options = odeset('RelTol',1e-4,'AbsTol',1e-4);
    [t1,x_gust] = ode45(@(t,x) PvtolSim(t, x, yref, utrimd, K, pvtol, Upsilon),t,x0,options);
    % calculate control
    x_gust = x_gust';  % convert to column vector of states
    y_gust = C*x_gust; % output
    e = yref-y_gust;   % output error
    u_gust = K*[e(1,:);-x_gust(2,:);e(2,:);-x_gust(4,:);e(3,:);-x_gust(6,:)]+utrim(1:2)';% control
    
    Omegal_gust(i,:) = (u_gust(1,:)+u_gust(2,:))/2; % left rotor speed
    Omegar_gust(i,:) = (u_gust(1,:)-u_gust(2,:))/2; % right rotor speed
    zpos_gust(i,:) = x_gust(1,:);                   % vertical position
    xpos_gust(i,:) = x_gust(3,:);                   % lateral position
    theta_gust(i,:) = x_gust(5,:);                  % pitch angle
    
end

%% plots

figure(16) % lateral position step response
%set(gca,'FontSize',11)
plot(t1,xpos_step(1,:),'-',t1,xpos_step(2,:),':',t1,xpos_step(3,:),'--',t1,xpos_step(4,:),'-.','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$x(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','southeast')
% save
if(printflag) save_pdf(gcf,'xStepResp'); end

figure(17) % pitch  step response
plot(t1,theta_step(1,:)*180/pi,'-', t1,theta_step(2,:)*180/pi,':',t1,theta_step(3,:)*180/pi,'--',t1,theta_step(4,:)*180/pi,'-.','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$\theta(t)$ (deg)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca; h99.XMinorTick='on'; h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','southeast')
% save
if(printflag) save_pdf(gcf,'thetaStepResp'); end

figure(18) % vertical position step response
%set(gca,'FontSize',11)
plot(t1,zpos_step(1,:),'-',t1,zpos_step(2,:),':',t1,zpos_step(3,:),'--',t1,zpos_step(4,:),'-.','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$z(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca; h99.XMinorTick='on'; h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','southeast')
% save
if(printflag) save_pdf(gcf,'zStepResp'); end

figure(19) % rotor speeds
plot(t1,Omegal_step(1,:) ,'-b', t1,Omegar_step(1,:) ,'-r'  ,...
    t1,Omegal_step(2,:) ,':b', t1,Omegar_step(2,:) ,':r'  ,...
    t1,Omegal_step(3,:) ,'--b', t1,Omegar_step(3,:) ,'--r'  ,...
    t1,Omegal_step(4,:) ,'-.b', t1,Omegar_step(4,:) ,'-.r'  ,'linewidth',lw)
h99=gca; ylim(h99,[140 160]); % change axes limits
h99.XMinorTick='on'; h99.YMinorTick='on';
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$\{\Omega_l(t),\Omega_r(t)\}$ (rad/s)','Interpreter' ,'Latex','fontsize',fs)
grid
g99=legend('$\Omega_l$, $\Upsilon=-5^\circ$','$\Omega_r$, $\Upsilon=-5^\circ$',...
    '$\Omega_l$, $\Upsilon=0^\circ$','$\Omega_r$, $\Upsilon=0^\circ$',...
    '$\Omega_l$, $\Upsilon=5^\circ$','$\Omega_r$, $\Upsilon=5^\circ$',...
    '$\Omega_l$, $\Upsilon=10^\circ$','$\Omega_r$, $\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',10,'Location','northeast')
% save
if(printflag) save_pdf(gcf,'OmegaStepResp'); end

figure(20) % lateral position gust response
plot(t1,xpos_gust(1,:),'-',t1,xpos_gust(2,:),':',t1,xpos_gust(3,:),'--',t1,xpos_gust(4,:),'-.','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$x(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca; h99.XMinorTick='on'; h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,...
    'Position',[0.68 0.66 0.2 0.2],...
    'Interpreter','latex',...
    'FontSize',12);
% save
if(printflag) save_pdf(gcf,'xGustResp'); end

figure(21) % pitch gust step response
plot(t1,theta_gust(1,:)*180/pi,'-', t1,theta_gust(2,:)*180/pi,':',t1,theta_gust(3,:)*180/pi,'--',t1,theta_gust(4,:)*180/pi,'-.','linewidth',lw)
%plot(t1,theta_gust(1,:)*180/pi,'-', t1,theta_gust(2,:)*180/pi,':',t1,theta_gust(3,:)*180/pi,'--','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$\theta(t)$ (deg)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca;
h99.XMinorTick='on';
h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','northeast')
% save
if(printflag) save_pdf(gcf,'thetaGustResp'); end

figure(22) % vertical position gust response
plot(t1,zpos_gust(1,:),'-',t1,zpos_gust(2,:),':',t1,zpos_gust(3,:),'--',t1,zpos_gust(4,:),'-.','linewidth',lw)
%plot(t1,zpos_gust(1,:),'-',t1,zpos_gust(2,:),':',t1,zpos_gust(3,:),'--',t1,zpos_gust(4,:),'-.','linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$z(t)$ (m)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca;
h99.XMinorTick='on';
h99.YMinorTick='on';
g99=legend('$\Upsilon=-5^\circ$','$\Upsilon=0^\circ$','$\Upsilon=5^\circ$','$\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','northeast')
% save
if(printflag) save_pdf(gcf,'zGustResp'); end


figure(23) % rotor speeds
plot(t1,Omegal_gust(1,:) ,'-b', t1,Omegar_gust(1,:) ,'-r'  ,...
    t1,Omegal_gust(2,:) ,':b', t1,Omegar_gust(2,:) ,':r'  ,...
    t1,Omegal_gust(3,:) ,'--b', t1,Omegar_gust(3,:) ,'--r'  ,...
    t1,Omegal_gust(4,:) ,'-.b', t1,Omegar_gust(4,:) ,'-.r'  ,'linewidth',lw)
% plot(t1,Omegal_gust(1,:) ,'-b', t1,Omegar_gust(1,:) ,'-r'  ,...
%     t1,Omegal_gust(2,:) ,':b', t1,Omegar_gust(2,:) ,':r'  ,...
%     t1,Omegal_gust(3,:) ,'--b', t1,Omegar_gust(3,:) ,'--r'    ,'linewidth',lw)
xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
ylabel('$\{\Omega_l(t),\Omega_r(t)\}$ (rad/s)','Interpreter' ,'Latex','fontsize',fs)
grid
h99=gca;
h99.XMinorTick='on';
h99.YMinorTick='on';
g99=legend('$\Omega_l$, $\Upsilon=-5^\circ$','$\Omega_r$, $\Upsilon=-5^\circ$',...
    '$\Omega_l$, $\Upsilon=0^\circ$','$\Omega_r$, $\Upsilon=0^\circ$',...
    '$\Omega_l$, $\Upsilon=5^\circ$','$\Omega_r$, $\Upsilon=5^\circ$',...
    '$\Omega_l$, $\Upsilon=10^\circ$','$\Omega_r$, $\Upsilon=10^\circ$');
set(g99,'Interpreter' ,'Latex','fontsize',10,'Location','northeast')
% save
if(printflag) save_pdf(gcf,'OmegaGustResp'); end

%% animation/video

if(1)    
    w=d*ones(size(t1)); % wind
    
    %v = VideoWriter('birotor.avi','MPEG-4');
    v = VideoWriter('pvtol','MPEG-4');
    %v = VideoWriter('birotor.avi','Uncompressed AVI');
    open(v);
    for i=1:length(Upsilonset)
        Upsilon=Upsilonset(i)*pi/180
        Visualize_pvtol(xpos_gust(i,:),-zpos_gust(i,:),theta_gust(i,:),-w*.4,Upsilon,v);
    end
    close(v);
    close(100);
end %(0)
