% Section6.m
%
% Generate figures for Section 6 of paper
% "The Effect of Rotor Tilt on the Stability and Gust Rejection Properties
%  of VTOL Multirotor Aircraft"
% by James Whidborne Arthur Mendez and Alastair Cooke

% (c) James Whidborne
% Cranfield University, 19 October 2020

disp(' ')
disp('-----------------------------------')
disp('6 Quadrotor Step and Gust Responses')
disp('-----------------------------------')
disp(' ')

for Upsilondeg=[0,10] % 2 values of rotor inclination angle (degrees)
    %% define quadrotor geometry
    Upsilon = Upsilondeg*pi/180;      % rotor inclination angle (rad)
    Upsilonstr = num2str(Upsilondeg); % for printing
    disp(['Upsilon = ',Upsilonstr,' deg '])
    [quad, ~, ~, rho] = Coefficients(Upsilon); % Draganfly XPro quadrotor parameters
    wind=[0;0;0];                     % wind disturbance
    
    %% trim & lin
    % options for trim residual minimization
    trimoptions = optimset('TolFun',1e-7,'TolX',1e-7 ,...
        'MaxFunEvals',20000,'MaxIter',20000,...
        'Display','on');
    
    % set initial trim points guess
    xtrim0=zeros(12,1); utrim0=148.1287*ones(4,1);
    
    [ A,Bomega,xtrim,Omegatrim ] = QuadrotorTrimLin( quad,wind,xtrim0,utrim0,trimoptions);
    %xtrim=xtrim % check trim
    %if norm(wind)<1e-9, xtrim(9)=psi0; end % in still air
    %  control allocation matrix Omega = CA u
    B = Bomega*quad.CA;
    
    % x = [xe ye ze u v w phi theta psi p q r]- state vector
    % y = [xe ye ze                 psi]
    C=[eye(3,3),zeros(3,9);
        zeros(1,8),1,zeros(1,3)]; % output matrix
    
    Aaug=[[A;-C],zeros(16,4)]; % augment for integral action
    Baug=[B;zeros(4,4)];
    
    %% Design LQR controller
    Q=eye(16); R=eye(4)/2; % weighting matrices
    
    Klqr=lqr(Aaug,Baug,Q,R); % optimal LQR regulator
    K1=Klqr(:,1:12);         % state feedback
    K2=Klqr(:,13:16);        % integral of error feedback
    if(0)
        %% linear closed loop check
        Acl=[A-B*K1,  -B*K2; -C,zeros(4,4)]; % closed loop A matrix
        Bcl=[zeros(12,4);eye(4)];% closed loop B matrix
        Ccl=[C,zeros(4,4)];      % closed loop C matrix
        eig(Acl)                 % eigenvalues check
        
        Tcl=ss(Acl,Bcl,Ccl,0);   % closed loop linear system
        figure(601), step(Tcl)   % linear closed loop step response
    end%(0)
    
    if(0)
        %% linearized nonlinear closed loop stability check
        t=0;
        yref=[0; 0 ;0; 0];
        f=@(x,u) Quadrotor(t,x,yref,Omegatrim,Klqr, quad, wind);
        
        xaug0=[xtrim;zeros(4,1)];              % initial state
        [A,B]=LinearizeAB(f,xaug0,Omegatrim);   % linearize
        eig(A)
    end%(0)
    
    %% nonlinear model simulation step response
    
    % initialize storage vectors
    t=0:0.01:15; %  time
    N=length(t);
    % lateral reference step response
    Omega_step = zeros(4,N);
    xpos_step = zeros(12,N);
    % gust step response
    Omega_gust = zeros(4,N);
    xpos_gust = zeros(12,N);
    xaug0=[xtrim;zeros(4,1)];              % initial state
    
    % 10 m x-demand step response
    yref=[10; 0; 0; 0];     % output reference
    options = odeset('RelTol',1e-6,'AbsTol',1e-6);
    [t1,xaug_step]=ode45(@(t,x)QuadrotorSim(t,x,yref,Omegatrim,Klqr, quad, wind),t,xaug0,options);
    xaug_step=xaug_step';  % convert to column vector of states
    
    %% plot xyz step resonse to 10 m  demand in x
    figure(26)
    p99=plot(t1,xaug_step(1:3,:),'linewidth',lw);
    set(p99(1),'LineStyle','-');
    set(p99(2),'LineStyle','--');
    set(p99(3),'LineStyle','-.');
    h99=gca; xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
    %ylim(h99,[-0.2 1.2]); % change axes limits
    ylabel('$\{x(t), y(t), z(t)\}$ (m)','Interpreter' ,'Latex','fontsize',fs)
    g99=legend('$x(t)$','$y(t)$','$z(t)$' );
    set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','east')
    title(['$\Upsilon = ',Upsilonstr,'$ deg'],'Interpreter' ,'Latex','fontsize',fs)
    grid on
    h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
    % save
    printfilename=['xyzStepResp',Upsilonstr];
    if(printflag) save_pdf(gcf,printfilename); end
    
    %% plot roll pitch yaw step response to 10 m  demand in x
    figure(27)
    p99=plot(t1,xaug_step(7:9,:)*180/pi, 'linewidth',lw);
    set(p99(1),'LineStyle','-');
    set(p99(2),'LineStyle','--');
    set(p99(3),'LineStyle','-.');
    h99=gca; ylim(h99,[-25 10]); % change axes limits
    xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
    ylabel('$\{\phi(t), \theta(t), \psi(t)\}$ (deg)','Interpreter' ,'Latex','fontsize',fs)
    title(['$\Upsilon = ',Upsilonstr,'$ deg'],'Interpreter' ,'Latex','fontsize',fs)
    grid on
    h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
    g99=legend('$\phi(t)$','$\theta(t)$','$\psi(t)$' );
    set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','southeast')
    % save
    printfilename=['rpyStepResp',Upsilonstr];
    if(printflag) save_pdf(gcf,printfilename); end
    
    %% controls response to 10 m  demand in x
    
    % calculate controls     
    for ii=1:size(xaug_step,2)
        
        x = xaug_step(1:12,ii);     % state of quadrotor
        psi = x(9);         % yaw angle
        % rotate x & y error and integrals to align with body axes (not
        % used in paper)
        %Rpsi=[ cos(psi),  sin(psi); % (not used)
        %    -sin(psi),  cos(psi)]; 
        Rotatex=eye(16); 
        % Rotatex(1:2,1:2)=Rpsi; Rotatex(13:14,13:14)=Rpsi;% (not used)
        
        %% calculate control
        y_e = yref-[x(1:3);psi];       % output error
        u  =  -Klqr*Rotatex*xaug_step(:,ii);         % control u
        Omega(:,ii) = quad.CA*u + Omegatrim; % add trim and allocate to rotor speeds
    end
    figure(603) % not presented in paper
    p99=plot(t1,Omega, 'linewidth',lw);
    set(p99(1),'LineStyle','-');
    set(p99(2),'LineStyle','--');
    set(p99(3),'LineStyle','-.');
    set(p99(4),'LineStyle',':');
    %h99=gca; ylim(h99,[-25 10]); % change axes limits
    xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
    ylabel('$\Omega(t)$ (rad/s)','Interpreter' ,'Latex','fontsize',fs)
    title(['$\Upsilon = ',Upsilonstr,'$ deg'],'Interpreter' ,'Latex','fontsize',fs)
    grid on
    h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
    g99=legend('$\Omega(1)$','$\Omega(2)$','$\Omega(3)$','$\Omega(4)$' );
    set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','southeast')
    % save
    printfilename=['OmegaStepResp',Upsilonstr];
    if(printflag) save_pdf(gcf,printfilename); end
    
    %% nonlinear model simulation gust step disturbance response
    xaug0=[xtrim;zeros(4,1)];              % initial stste
    %xref=zeros(12,1);      % state reference
    yref=[0;0;0;0];          % output reference
    wind=[10;0;0];            % wind
    options = odeset('RelTol',1e-4,'AbsTol',1e-4);
    t=0:0.01:15 ; %  time
    [t1,xaug_gust]=ode45(@(t,x)QuadrotorSim(t,x,yref,Omegatrim,Klqr, quad, wind),t,xaug0,options);
    % calculate control
    xaug_gust=xaug_gust';  % convert to column vector of states
    Omega_gust = -quad.CA*(Klqr*xaug_gust) + Omegatrim ; % control
    
    %% plot xyz response to gust disturbance
    figure(28)
    p99=plot(t1,xaug_gust(1:3,:),'linewidth',lw);
    set(p99(1),'LineStyle','-');
    set(p99(2),'LineStyle','--');
    set(p99(3),'LineStyle','-.');
    grid on
    h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
    xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
    ylim(h99,[-1.0 1.5]); % change axes limits
    ylabel('$\{x(t), y(t), z(t)\}$ (m)','Interpreter' ,'Latex','fontsize',fs)
    g99=legend('$x(t)$','$y(t)$','$z(t)$' );
    set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','northeast')
    title(['$\Upsilon = ',Upsilonstr,'$ deg'],'Interpreter' ,'Latex','fontsize',fs)
    % save
    printfilename=['xyzGustResp',Upsilonstr];
    if(printflag) save_pdf(gcf,printfilename); end
    
    %% plot roll pitch yaw response to gust disturbance
    figure(29)
    p99=plot(t1,xaug_gust(7:9,:)*180/pi, 'linewidth',lw);
    set(p99(1),'LineStyle','-');
    set(p99(2),'LineStyle','--');
    set(p99(3),'LineStyle','-.');
    h99=gca; h99.XMinorTick='on';h99.YMinorTick='on';
    ylim(h99,[-5 30]); % change axes limits
    xlabel('$t$ (s)','Interpreter' ,'Latex','fontsize',fs)
    ylabel('$\{\phi(t), \theta(t), \psi(t)\}$ (deg)','Interpreter' ,'Latex','fontsize',fs)
    title(['$\Upsilon = ',Upsilonstr,'$ deg'],'Interpreter' ,'Latex','fontsize',fs)
    grid on
    g99=legend('$\phi(t)$','$\theta(t)$','$\psi(t)$' );
    set(g99,'Interpreter' ,'Latex','fontsize',12,'Location','northeast')
    % save
    printfilename=['rpyGustResp',Upsilonstr]; % roll pitch yaw
    if(printflag) save_pdf(gcf,printfilename); end
    
    disp('Press any key to continue ...')
    pause
end