function [T,H,Q]=ThrustDragMap(Omega,alpha,V,rotor,rho)
%THRUSTDRAGMAP
%    [T,H,Q]=ThrustDragMap(Omega,alpha,V,rotor,rho)
%    Rotor thrust, drag, and blade torque maps as functions of Omega, AoA and V
%
%    T : rotor thrust map
%    H : horizontal rotor drag map
%    Q : Rotor blade torque  map
%
%    Omega : rotor speed (rad/s)
%    alpha : rotor disc angle of attack (rad)
%    V :  airspeed (m/s) 
%    rho :   air density (kg/m3)
%    rotor : rotor data structure (see Coefficients.m)
%
 
%   (c) James Whidborne
%   Cranfield University, 15 August 2020
   
n_alpha=length(alpha);
n_V=length(V);
n_Omega=length(Omega);
mass=2.356;          % vehicle mass[kg]
T0=mass*9.81/4;      % approx thrust for fzero search initial value
A= pi*rotor.R^2;     % disc area
T=zeros(n_V,n_Omega,n_alpha); H=T;Q=T; % pre-allocate arrays
for i=1:n_alpha
    for j=1:n_Omega
        for k=1:n_V
            T(k,j,i) = fzero(@(x)rho*A*(Omega(j)*rotor.R)^2*RotorCT(x,alpha(i),V(k),Omega(j),rho,rotor)-x, T0);
            H(k,j,i) = rho*A*(Omega(j)*rotor.R)^2*RotorCH(T(k,j,i),alpha(i),V(k),Omega(j),rho,rotor);
            Q(k,j,i) = rho*A*Omega(j)^2*rotor.R^3*RotorCQ(T(k,j,i),alpha(i),V(k),Omega(j),rho,rotor);
        end
    end
end
return

%% if spline approximations are needed, use below for example
%spT = spapi({1,1,1},[4 6 8],{V,Omega,alpha},T);
%spH = spapi({1,3,1},[5 6 11],{V,Omega,alpha},H);